@@ -46,6 +46,8 @@ ALL_TASK = {
     "cpm_lm": ("flagai.model.gpt2_model", "GPT2Model"),
     "t5_seq2seq": ["flagai.model.t5_model", "T5Model"],
     "t5_lm": ["flagai.model.t5_model", "T5Model"],
+    "t5_title-generation": ["flagai.model.t5_model", "T5Model"],
+    "alm_lm": ["flagai.model.alm_model", "ALMModel"],
     "glm_lm": ["flagai.model.glm_model", "GLMModel"],
     "glm_seq2seq": ["flagai.model.glm_model", "GLMForSeq2Seq"],
     "glm_poetry": ["flagai.model.glm_model", "GLMForSeq2Seq"],
@@ -54,13 +56,14 @@ ALL_TASK = {
     "glm_title-generation": ["flagai.model.glm_model", "GLMForSeq2Seq"],
     "opt_seq2seq": ("flagai.model.opt_model", "OPTModel"),
     "opt_lm": ("flagai.model.opt_model", "OPTModel"),
+    "galactica_lm": ("flagai.model.galactica_model", "GalacticaModel"),
     "vit_classification": ("flagai.model.vision.vit", "VisionTransformer"),
     "clip_txt_img_matching": ("flagai.model.mm.clip_model", "CLIP"),
     "swinv1_classification": ("flagai.model.vision.swinv1", "SwinTransformer"),
     "swinv2_classification": ("flagai.model.vision.swinv2",
                               "SwinTransformerV2"),
     "cpm3_lm": ("flagai.model.cpm3_model", "CPM3"),
-    "cpm3_trian": ("flagai.model.cpm3_trian_model", "CPM3"),
+    "cpm3_train": ("flagai.model.cpm3_train_model", "CPM3"),
     "diffusion_text2img": ("flagai.model.mm.AltDiffusion", "LatentDiffusion"),
     "altclip_txt_img_matching": ("flagai.model.mm.AltCLIP", "AltCLIP"),
     "evaclip_txt_img_matching": ("flagai.model.mm.eva_clip_model", "EVA_CLIP"),
@@ -73,9 +76,11 @@ MODEL_DICT = {
     "t5-base-en": ["flagai.model.t5_model", "T5Model", "t5", "nlp"],
     "t5-base-ch": ["flagai.model.t5_model", "T5Model", "t5", "nlp"],
     "glm-large-ch": ["flagai.model.glm_model", "GLMModel", "glm", "nlp"],
-    "alm-1.0": ["flagai.model.glm_model", "GLMModel", "glm", "nlp"],
+    "alm-1.0": ["flagai.model.alm_model", "ALMModel", "alm", "nlp"],
     "glm-large-en": ["flagai.model.glm_model", "GLMModel", "glm", "nlp"],
+    "glm-large-en-generation": ["flagai.model.glm_model", "GLMModel", "glm", "nlp"],
     "gpt2-base-ch": ["flagai.model.gpt2_model", "GPT2Model", "gpt2", "nlp"],
+    "gpt2-base-en": ["flagai.model.gpt2_model", "GPT2Model", "gpt2", "nlp"],
     "cpm-large-ch": ["flagai.model.gpt2_model", "GPT2Model", "cpm", "nlp"],
     "opt-125m-en": ["flagai.model.opt_model", "OPTModel", "opt", "nlp"],
     "opt-350m-en": ["flagai.model.opt_model", "OPTModel", "opt", "nlp"],
@@ -88,6 +93,10 @@ MODEL_DICT = {
     "glm-10b-ch": ["flagai.model.glm_model", "GLMModel", "glm", "nlp"],
     "cpm3": ["flagai.model.cpm3_model", "CPM3", "cpm3", "nlp"],
     "cpm3-train": ["flagai.model.cpm3_train_model", "CPM3", "cpm3", "nlp"],
+    "galactica-1.3b-en": ["flagai.model.galactica_model", "GalacticaModel", "galactica", "nlp", "flagai.data.tokenizer.galactica.galactica_tokenizer", "GalacticaTokenizer"],
+    "galactica-6.7b-en": ["flagai.model.galactica_model", "GalacticaModel", "galactica", "nlp", "flagai.data.tokenizer.galactica.galactica_tokenizer", "GalacticaTokenizer"],
+    "galactica-30b-en": ["flagai.model.galactica_model", "GalacticaModel", "galactica", "nlp", "flagai.data.tokenizer.galactica.galactica_tokenizer", "GalacticaTokenizer"],
+    "galactica-120b-en": ["flagai.model.galactica_model", "GalacticaModel", "galactica", "nlp", "flagai.data.tokenizer.galactica.galactica_tokenizer", "GalacticaTokenizer"],
     "vit-base-p16-224":
         ["flagai.model.vision.vit", "VisionTransformer", "vit", "vision"],
     "vit-base-p16-384":
@@ -129,6 +138,7 @@ MODEL_DICT = {
     "altclip-bert-b": ["flagai.models.mm.AltCLIP", "AltCLIP", "altclip", "mm", "flagai.model.mm.AltCLIP",
                        "AltCLIPProcessBert"],
     "eva-clip": ["flagai.model.mm.eva_clip_model", "EVA_CLIP", "evaclip", "mm"],
+
 }
 
 
@@ -167,9 +177,7 @@ class AutoLoader:
                                          class_num=2)
 
         """
-
         raw_model_name = copy.deepcopy(model_name)
-
         model_name = model_name.lower()
 
         if model_name not in MODEL_DICT:
@@ -179,7 +187,6 @@ class AutoLoader:
 
         brief_model_name = MODEL_DICT[model_name][2]
         model_type = MODEL_DICT[model_name][3]
-
         # The dir to save config, vocab and model.
 
         self.model_name = ALL_TASK.get(f"{brief_model_name}_{task_name}", None)
@@ -194,13 +201,7 @@ class AutoLoader:
 
         download_path = os.path.join(model_dir, raw_model_name)
         print("*" * 20, task_name, model_name)
-
-        model_id = _get_model_id(f"{raw_model_name}-{task_name}")
-        if model_id != 'null':
-            model_name_ = f"{raw_model_name}-{task_name}"
-        else:
-            model_name_ = raw_model_name
-
+        model_name_ = self.is_exist_finetuned_model(raw_model_name, task_name)
         self.model = getattr(LazyImport(self.model_name[0]),
                              self.model_name[1]).from_pretrain(
             download_path=model_dir,
@@ -208,12 +209,18 @@ class AutoLoader:
             only_download_config=only_download_config,
             device=device,
             **kwargs)
+        if kwargs.get("use_fp16", None):
+            self.model.half()
 
         if model_type == "nlp":
-            tokenizer_class = getattr(LazyImport("flagai.data.tokenizer"),
-                                      "Tokenizer")
-            self.tokenizer = tokenizer_class.from_pretrained(
-                model_name, cache_dir=download_path)
+            if brief_model_name in ["galactica", ]:
+                self.tokenizer = getattr(LazyImport(MODEL_DICT[model_name][4]),
+                                                    MODEL_DICT[model_name][5])(download_path)
+            else :
+                tokenizer_class = getattr(LazyImport("flagai.data.tokenizer"),
+                                        "Tokenizer")
+                self.tokenizer = tokenizer_class.from_pretrained(
+                    model_name, cache_dir=download_path)
 
         elif model_type == "mm":
             if model_name.startswith("altdiffusion"):
@@ -226,6 +233,7 @@ class AutoLoader:
                 self.tokenizer = ClipTokenizer(bpe_path=os.path.join(download_path, 'bpe_simple_vocab_16e6.txt.gz'))
                 self.transform = None
             else:
+                
                 self.process = getattr(LazyImport(MODEL_DICT[model_name][4]),
                                        MODEL_DICT[model_name][5]).from_pretrained(
                     os.path.join(model_dir, raw_model_name))
@@ -236,6 +244,19 @@ class AutoLoader:
             self.tokenizer = None
             self.transform = None
 
+    def is_exist_finetuned_model(self, raw_model_name, task_name):
+        try:
+            model_id = _get_model_id(f"{raw_model_name}-{task_name}")
+            if model_id != 'null':
+                model_name_ = f"{raw_model_name}-{task_name}"
+                return model_name_
+            else :
+                return raw_model_name
+
+        except:
+            print("Model hub is not reachable.")
+            return raw_model_name
+
     def get_task_name(self, brief_model_name):
         all_model_task = list(ALL_TASK.keys())
         model_tasks = [t for t in all_model_task if brief_model_name in t]
