@@ -141,11 +141,12 @@ class SummmaryProcessor:
                                    text_a=source_text,
                                    text_b=target_text,
                                    meta=meta)
-            if idx < 10:
+            if idx < 1:
                 print_rank_0(
                     (source_text.encode('utf-8'), target_text.encode('utf-8'),
                      meta["ref"].encode('utf-8')))
             example_list.append(example)
+        
         return example_list
 
 
@@ -393,7 +394,6 @@ class Seq2SeqDataset(torch.utils.data.Dataset):
         self.dataset_type = dataset_type
         self.tokenizer = tokenizer
         self.dataset_name = dataset_type
-
         if not os.path.exists(data_dir + '/' + task_name):
             SuperGlueProcessor()._download_data(data_dir, task_name)
 
@@ -477,7 +477,7 @@ class ExtractionDataset(torch.utils.data.Dataset):
     def __getitem__(self, idx):
         example = self.example_list[idx]
         source_text, target_text = example.text_a, example.text_b
-        mask_token = 'MASK'
+        mask_token = 'mask'
         mask_id = self.tokenizer.get_command_id(mask_token)
         sop_id = self.tokenizer.get_command_id('sop')
         eop_id = self.tokenizer.get_command_id('eop')
@@ -612,7 +612,7 @@ class BlankLMDataset(torch.utils.data.Dataset):
     def __getitem__(self, idx):
         example = self.example_list[idx]
         source_text = example.text_a
-        mask_token = 'gMASK' if self.args.task_mask else 'MASK'
+        mask_token = 'gMASK' if self.args.task_mask else 'mask'
         mask_id = self.tokenizer.get_command_id(mask_token)
         sop_id = self.tokenizer.get_command_id('sop')
         eop_id = self.tokenizer.get_command_id('eop')
