@@ -37,7 +37,7 @@ class CPMTokenizer(object):
         self.encoder = json.load(open(vocab_file))
         self.decoder = {v: k for k, v in self.encoder.items()}
 
-        self.sp = spm.SentencePieceProcessor(model_file=model_file)
+        self.sp_model = spm.SentencePieceProcessor(model_file=model_file)
         self.translator = str.maketrans(" \n", "\u2582\u2583")
         self.token_start_id = 0
         self.token_end_id = 3
@@ -48,6 +48,13 @@ class CPMTokenizer(object):
     def vocab_size(self):
         return len(self.encoder)
 
+    def get_vocab(self):
+        vocab = {
+            self.convert_id_to_token(i): i
+            for i in range(self.vocab_size)
+        }
+        return vocab
+    
     def __len__(self):
         return len(self.encoder) + len(self.special_tokens)
 
@@ -57,19 +64,28 @@ class CPMTokenizer(object):
 
     def tokenize(self, text):
         """ Tokenize a string. """
-        seg_list = [
-            x.translate(self.translator)
-            for x in jieba.cut(text, cut_all=False)
-        ]
-        new_seg = " ".join(seg_list)
-        return self.sp.encode(new_seg)
+        seg_list = [x.translate(self.translator) for x in jieba.cut(text, cut_all=False)]
+        new_seg = "".join(seg_list)
+        return self.sp_model.encode(new_seg)
 
     def encode(self, text):
         res = self.tokenize(text)
         return res
+    
+    def convert_tokens_to_ids(self, tokens):
+        return [self.sp_model.PieceToId(token) for token in tokens]
+
+    def convert_token_to_id(self, token):
+        return self.sp_model.PieceToId(token)
+
+    def convert_id_to_token(self, idx):
+        return self.sp_model.IdToPiece(int(idx))
+
+    def convert_ids_to_tokens(self, idxs):
+        return [self.sp_model.IdToPiece(int(idx)) for idx in idxs]
 
     def decode(self, tokens):
-        text = self.sp.decode(tokens)
+        text = self.sp_model.decode(tokens)
         text = text.replace(' ', '').replace('\u2582',
                                              ' ').replace('\u2583', '\n')
         return text
@@ -78,3 +94,18 @@ class CPMTokenizer(object):
         res = self.encode(text)
 
         return {"input_ids": res}
+    
+    def convert_tokens_to_string(self, tokens, all_command_token={}):
+        """Converts a sequence of tokens (string) in a single string."""
+        current_sub_tokens = []
+        out_string = ""
+        for token in tokens:
+            # make sure that special tokens are not decoded using sentencepiece model
+            if token in all_command_token:
+                out_string += self.sp_model.decode_pieces(
+                    current_sub_tokens) + token + " "
+                current_sub_tokens = []
+            else:
+                current_sub_tokens.append(token)
+        out_string += self.sp_model.decode_pieces(current_sub_tokens)
+        return out_string.strip()
