@@ -60,7 +60,7 @@ class GLM10bENBPETokenizer(GLMTokenizer):
                              self.text_tokenizer.encoder['</s>']),
                 CommandToken('cls', '[CLS]',
                              self.text_tokenizer.encoder['<s>']),
-                CommandToken('MASK',
+                CommandToken('mask',
                              '[MASK]',
                              self.text_tokenizer.encoder['<mask>'],
                              lstrip=True),
@@ -88,7 +88,7 @@ class GLM10bENBPETokenizer(GLMTokenizer):
                     CommandToken('sop', '<|startofpiece|>', self.num_tokens),
                     CommandToken('eop', '<|endofpiece|>', self.num_tokens + 1),
                     CommandToken('cls', '[CLS]', self.num_tokens + 2),
-                    CommandToken('MASK',
+                    CommandToken('mask',
                                  '[MASK]',
                                  self.num_tokens + 3,
                                  lstrip=True),
