@@ -44,8 +44,8 @@ class T5BPETokenizer(Tokenizer):
             CommandToken('sep', '[SEP]', self.num_tokens),
 
             CommandToken('pad', '[PAD]', self.num_tokens + 1),
-            CommandToken('ENC', '[CLS]', self.num_tokens + 2),
-            CommandToken('MASK', '[MASK]',
+            CommandToken('cls', '[CLS]', self.num_tokens + 2),
+            CommandToken('mask', '[MASK]',
                          self.num_tokens + 3),
         ]
         self._command_tokens.extend([
