@@ -77,9 +77,11 @@ class BaseModel(Module):
         config_path = os.path.join(download_path, "config.json")
         checkpoint_path = os.path.join(download_path, "pytorch_model.bin")
 
-        def load_local(checkpoint_path):
+        def load_local(checkpoint_path, only_download_config=False):
             model = cls.init_from_json(config_path, **kwargs)
             model.to(device)
+            if only_download_config:
+                return model
             if os.getenv('ENV_TYPE') != 'deepspeed+mpu':
                 if os.path.exists(checkpoint_path):
                     model.load_weights(checkpoint_path)
@@ -115,7 +117,7 @@ class BaseModel(Module):
                     model.load_weights(checkpoint_path)
             return model
 
-        def load_diffusion_local(yaml_path, only_download_config=False):
+        def load_diffusion_local(yaml_path, only_download_config=False, **kwargs):
             """
             Now only diffusion models requires yaml
             """
@@ -126,8 +128,8 @@ class BaseModel(Module):
             config = OmegaConf.load(f"{yaml_path}")
             model_config = config.model
             model_config.params.cond_stage_config.params.download_path = raw_download_path
-
-            model = cls(**model_config.get("params", dict()))
+            kwargs.update(model_config.get("params", dict()))
+            model = cls(**kwargs)
             if not only_download_config:
                 model = cls._load_state_dict_into_model(
                     model,
@@ -140,13 +142,13 @@ class BaseModel(Module):
             """
             Now only diffusion models requires yaml
             """
-            return load_diffusion_local(yaml_path, only_download_config=only_download_config)
+            return load_diffusion_local(yaml_path, only_download_config=only_download_config, **kwargs)
         elif os.path.exists(config_path):
             """
             It is fine when checkpoint_path does not exist, for the case that only_download_config=True
             At that time the model will not be loaded. 
             """
-            return load_local(checkpoint_path)
+            return load_local(checkpoint_path, only_download_config=only_download_config)
 
         try:
             model_id = _get_model_id(model_name)
