@@ -176,6 +176,7 @@ class VocabParallelEmbedding(torch.nn.Module):
         if os.getenv('ENV_TYPE') == 'deepspeed+mpu':
             self.weight.model_parallel = True
         # And initialize.
+
         _initialize_affine_weight(self.weight, self.num_embeddings,
                                   self.embedding_dim,
                                   self.num_embeddings_per_partition, 0,
@@ -193,6 +194,7 @@ class VocabParallelEmbedding(torch.nn.Module):
                                       self.padding_idx, self.max_norm,
                                       self.norm_type, self.scale_grad_by_freq,
                                       self.sparse)
+
         # Mask the output embedding.
         output_parallel[input_mask, :] = 0.0
         # Reduce across all the model parallel GPUs.
@@ -310,6 +312,7 @@ class BertEmbeddings(nn.Module):
         embeddings = self.dropout(embeddings)
         return embeddings
 
+
 class CPM3Embedding(torch.nn.Module):
     r"""Embed a sequence of indices through a embedding lookup matrix :math:`\mathbf{W}`.
 
@@ -320,22 +323,23 @@ class CPM3Embedding(torch.nn.Module):
         init_mean (float, optional): mean of :math:`\mathbf{W}\sim\mathcal{N}(\text{mean}, \text{std}^2)`. Defaults to 0.
         init_std (float, optional): std of :math:`\mathbf{W}\sim\mathcal{N}(\text{mean}, \text{std}^2)`. Defaults to 1.
     """
-    def __init__(self,
-                 vocab_size : int,
-                 embedding_size : int,
-                 length_scale : bool = False,
-                 dtype = torch.half,
-                 int8 :bool = False,
-                ):
+
+    def __init__(
+        self,
+        vocab_size: int,
+        embedding_size: int,
+        length_scale: bool = False,
+        dtype=torch.half,
+        int8: bool = False,
+    ):
         super().__init__()
         self.dim_model = embedding_size
         self.weight = torch.nn.Parameter(
-            torch.empty(vocab_size, embedding_size, dtype=dtype),
-        )
+            torch.empty(vocab_size, embedding_size, dtype=dtype), )
         self.length_scale = length_scale
         self.int8 = int8
 
-    def forward(self, ids : torch.Tensor):
+    def forward(self, ids: torch.Tensor):
         """ 
         Args:
             ids (:obj:`torch.Tensor` of shape ``(batch_size, seq_len)``): Indices of input sequence tokens.
@@ -343,13 +347,13 @@ class CPM3Embedding(torch.nn.Module):
         Return:
             :obj:`torch.Tensor` of shape ``(batch_size, seq_len, embedding_size)``: The embedding output.
         """
-        
+
         embeds = F.embedding(ids, self.weight)
         if self.length_scale:
             embeds = embeds / math.sqrt(self.dim_model)
         return embeds
-    
-    def projection(self, x : torch.Tensor):
+
+    def projection(self, x: torch.Tensor):
         """
         Projection based on embedding's weight. For example, embedding map vocab_size to embed_size, than projection map embed_size back to vocab_size.
 
@@ -366,18 +370,19 @@ class CPM3Embedding(torch.nn.Module):
 
 class CPM3SegmentPositionEmbedding(torch.nn.Module):
 
-    def __init__(self, num_heads, 
-    	               num_segments = 1,
-                       num_buckets = 32, 
-                       max_distance = 128, 
-                       max_exact_rate = 0.25,
-                       max_distance_rate = 1.0,
-                       bidirectional = False, 
-                       dtype = torch.half,
-                       absolute_inner_segment = True):
+    def __init__(self,
+                 num_heads,
+                 num_segments=1,
+                 num_buckets=32,
+                 max_distance=128,
+                 max_exact_rate=0.25,
+                 max_distance_rate=1.0,
+                 bidirectional=False,
+                 dtype=torch.half,
+                 absolute_inner_segment=True):
 
         super().__init__()
-        
+
         self.num_heads = num_heads
         self.num_buckets = num_buckets
         self.max_distance = max_distance
@@ -391,10 +396,15 @@ class CPM3SegmentPositionEmbedding(torch.nn.Module):
         #     init_method = bmt.ParameterInitializer(torch.nn.init.normal_, mean=0.0, std=0.0)
         # )
         self.relative_attention_bias = torch.nn.Parameter(
-            torch.empty(num_segments * num_segments + num_buckets, num_heads, dtype=dtype)
-        )
+            torch.empty(num_segments * num_segments + num_buckets,
+                        num_heads,
+                        dtype=dtype))
 
-    def forward(self, key_pos = None, query_pos = None, key_segment = None, query_segment = None):
+    def forward(self,
+                key_pos=None,
+                query_pos=None,
+                key_segment=None,
+                query_segment=None):
         """
         Args:
             key_len: int
@@ -403,13 +413,14 @@ class CPM3SegmentPositionEmbedding(torch.nn.Module):
             out : (batch_size, num_heads, query_len, key_len)   fp16
         """
         with torch.no_grad():
-        
+
             batch = key_pos.size(0)
             keylen = key_pos.size(1)
             querylen = query_pos.size(1)
 
             assert key_pos.size(0) == query_pos.size(0)
-            assert keylen == key_segment.size(1) and querylen == query_segment.size(1)
+            assert keylen == key_segment.size(
+                1) and querylen == query_segment.size(1)
 
             key_pos = key_pos.view(batch, -1, keylen)
             query_pos = query_pos.view(batch, querylen, -1)
@@ -423,7 +434,8 @@ class CPM3SegmentPositionEmbedding(torch.nn.Module):
             #     max_exact = self.max_exact_rate,
             #     max_distance = self.max_distance_rate
             # )
-            relative_position_bucket = self._segment_relative_position_bucket(query_segment, key_segment)
+            relative_position_bucket = self._segment_relative_position_bucket(
+                query_segment, key_segment)
             relative_position_bucket = relative_position_bucket + self.num_buckets  # 与相对位置编码区间不重叠
 
             # bucket_segment = (key_segment != query_segment) * (key_segment + query_segment * (self.num_segments - 1) + (query_segment >= key_segment))
@@ -431,54 +443,92 @@ class CPM3SegmentPositionEmbedding(torch.nn.Module):
             # b*q*k
             if self.absolute_inner_segment:
                 absolute_position_bucket = self._absolute_position_bucket(
-                    torch.arange(keylen, dtype=torch.int32, device=relative_position_bucket.device)[None, :] - torch.arange(querylen, dtype=torch.int32, device=relative_position_bucket.device)[:, None],
+                    torch.arange(
+                        keylen,
+                        dtype=torch.int32,
+                        device=relative_position_bucket.device)[None, :] -
+                    torch.arange(querylen,
+                                 dtype=torch.int32,
+                                 device=relative_position_bucket.device)[:,
+                                                                         None],
                     bidirectional=self.bidirectional,
                     num_buckets=self.num_buckets,
-                    max_distance = self.max_distance
-                )
-                relative_position_bucket = torch.where((key_segment == query_segment), absolute_position_bucket[None, :, :], relative_position_bucket)
+                    max_distance=self.max_distance)
+                relative_position_bucket = torch.where(
+                    (key_segment == query_segment),
+                    absolute_position_bucket[None, :, :],
+                    relative_position_bucket)
             # (batch, len_q, len_k)
- 
+
         # (batch, len_q, len_k, num_heads)
-        embeds = F.embedding(relative_position_bucket, self.relative_attention_bias)
+        embeds = F.embedding(relative_position_bucket,
+                             self.relative_attention_bias)
         # (batch, num_heads, len_q, len_k)
         embeds = embeds.permute(0, 3, 1, 2).contiguous()
         return embeds
 
-    def _relative_position_bucket(self, relative_position, bidirectional=True, num_buckets = 32, max_exact=0.125, max_distance=0.5):
+    def _relative_position_bucket(self,
+                                  relative_position,
+                                  bidirectional=True,
+                                  num_buckets=32,
+                                  max_exact=0.125,
+                                  max_distance=0.5):
         relative_buckets = 0
         if bidirectional:
             num_buckets //= 2
             max_exact /= 2
-            relative_buckets = (relative_position > 0).to(torch.int32) * num_buckets
+            relative_buckets = (relative_position > 0).to(
+                torch.int32) * num_buckets
             relative_position = torch.abs(relative_position)
         else:
-            relative_position = -torch.min(relative_position, torch.zeros_like(relative_position))
+            relative_position = -torch.min(relative_position,
+                                           torch.zeros_like(relative_position))
         max_exact /= 2
         is_small = relative_position < max_exact
         half_num_buckets = num_buckets // 2
-        relative_postion_if_large = half_num_buckets + (torch.log(relative_position / max_exact) / math.log(max_distance / max_exact) * (num_buckets - half_num_buckets)).to(torch.int32)
-        relative_postion_if_large = torch.min(relative_postion_if_large, torch.full_like(relative_postion_if_large, num_buckets - 1))
-        relative_buckets += torch.where(is_small, (relative_position / max_exact * half_num_buckets).to(torch.int32), relative_postion_if_large)
+        relative_postion_if_large = half_num_buckets + (
+            torch.log(relative_position / max_exact) /
+            math.log(max_distance / max_exact) *
+            (num_buckets - half_num_buckets)).to(torch.int32)
+        relative_postion_if_large = torch.min(
+            relative_postion_if_large,
+            torch.full_like(relative_postion_if_large, num_buckets - 1))
+        relative_buckets += torch.where(is_small,
+                                        (relative_position / max_exact *
+                                         half_num_buckets).to(torch.int32),
+                                        relative_postion_if_large)
         return relative_buckets
 
     def _segment_relative_position_bucket(self, query_segment, key_segment):
         """
-            segment1看segment2的所有位置编码相同，看segment3的位置编码也相同（是另一个值）
+            All positional encodings of segment2 are the same in the view of segment1. Same for segment3 and segment1(but is a different value)
         """
         return query_segment * self.num_segments + key_segment
 
-    def _absolute_position_bucket(self, relative_position, bidirectional=True, num_buckets=32, max_distance=128):
+    def _absolute_position_bucket(self,
+                                  relative_position,
+                                  bidirectional=True,
+                                  num_buckets=32,
+                                  max_distance=128):
         relative_buckets = 0
         if bidirectional:
             num_buckets //= 2
-            relative_buckets = (relative_position > 0).to(torch.int32) * num_buckets
+            relative_buckets = (relative_position > 0).to(
+                torch.int32) * num_buckets
             relative_position = torch.abs(relative_position)
         else:
-            relative_position = -torch.min(relative_position, torch.zeros_like(relative_position))
+            relative_position = -torch.min(relative_position,
+                                           torch.zeros_like(relative_position))
         max_exact = num_buckets // 2
         is_small = relative_position < max_exact
-        relative_postion_if_large = max_exact + (torch.log(relative_position.float() / max_exact) / math.log(max_distance / max_exact) * (num_buckets - max_exact)).to(torch.int32)
-        relative_postion_if_large = torch.min(relative_postion_if_large, torch.full_like(relative_postion_if_large, num_buckets - 1))
-        relative_buckets += torch.where(is_small, relative_position.to(torch.int32), relative_postion_if_large)
+        relative_postion_if_large = max_exact + (
+            torch.log(relative_position.float() / max_exact) /
+            math.log(max_distance / max_exact) *
+            (num_buckets - max_exact)).to(torch.int32)
+        relative_postion_if_large = torch.min(
+            relative_postion_if_large,
+            torch.full_like(relative_postion_if_large, num_buckets - 1))
+        relative_buckets += torch.where(is_small,
+                                        relative_position.to(torch.int32),
+                                        relative_postion_if_large)
         return relative_buckets
