@@ -18,6 +18,10 @@ from flagai.mpu.mappings import reduce_from_model_parallel_region
 from flagai.mpu.mappings import scatter_to_model_parallel_region
 from flagai.mpu.utils import divide
 
+from flagai.model.layers.linear import CPM3Linear
+
+from .layer_norm import CPM3LayerNorm
+
 
 def _initialize_affine_weight(weight,
                               output_size,
@@ -412,3 +416,294 @@ class RowParallelLinear(torch.nn.Module):
         else:
             output = output_
         return output
+
+
+class DenseGatedACT(torch.nn.Module):
+
+    def __init__(self,
+                 dim_in : int,
+                 dim_ff : int,
+                 activate_fn : str = "gelu",
+                 dtype = torch.half,
+                 int8 = False,
+                 init_mean = 0.0,
+                 init_std = 0.02,
+                 bias = False,
+                 length_scale : bool = False,
+        ):
+        super().__init__()
+
+        self.w_0 = CPM3Linear(
+            dim_in = dim_in,
+            dim_out = dim_ff,
+            length_scale = length_scale,
+            length_scale_before = False,
+            dtype = dtype,
+            int8 = int8,
+            bias = bias,
+        )
+
+        self.w_1 = CPM3Linear(
+            dim_in = dim_in,
+            dim_out = dim_ff,
+            length_scale = length_scale,
+            length_scale_before = False,
+            dtype = dtype,
+            int8 = int8,
+            bias = bias,
+        )
+
+        if activate_fn == "relu":
+            self.act = torch.nn.ReLU()
+        elif activate_fn == "gelu":
+            self.act = torch.nn.GELU()
+        else:
+            raise ValueError("Unsupported activation function: %s" % (activate_fn))
+    
+    def forward(self, x : torch.Tensor):
+        """ This model inherits from bmt.DistributedModule. 
+            Transform an input tensor from one feature space to another via a nonlinear operation
+        
+        Args:
+            x (:obj:`torch.Tensor` of shape ``(batch, seq_len, dim_in)``): Tensor that will be subject to nonlinear operations.
+
+        Return:
+            out (:obj:`torch.Tensor` of shape ``(batch, seq_len, dim_ff)``) 
+
+        """
+        gelu_score = self.act( self.w_0(x) )
+        x = self.w_1(x)
+
+        x = gelu_score * x
+        return x
+
+
+class DenseACT(torch.nn.Module):
+
+    def __init__(self,
+                 dim_in : int,
+                 dim_ff : int,
+                 activate_fn : str = "gelu",
+                 dtype = torch.half,
+                 int8 = False,
+                 init_mean = 0.0,
+                 init_std = 0.02,
+                 bias = False,
+                 length_scale : bool = False,
+        ):
+        super().__init__()
+
+        self.w = CPM3Linear(
+            dim_in = dim_in,
+            dim_out = dim_ff,
+            length_scale = length_scale,
+            length_scale_before = False,
+            dtype = dtype,
+            int8 = int8,
+            bias = bias,
+        )
+        
+        if activate_fn == "relu":
+            self.act = torch.nn.ReLU()
+        elif activate_fn == "gelu":
+            self.act = torch.nn.GELU()
+        else:
+            raise ValueError("Unsupported activation function: %s" % (activate_fn))
+
+    def forward(self, x : torch.Tensor):
+        """ This model inherits from bmt.DistributedModule. 
+            Transform an input tensor from one feature space to another via a nonlinear operation
+        
+        Args:
+            x (:obj:`torch.Tensor` of shape ``(batch, seq_len, dim_in)``): Tensor that will be subject to nonlinear operations.
+
+        Return:
+            out (:obj:`torch.Tensor` of shape ``(batch, seq_len, dim_ff)``) 
+        """
+        x = self.w(x)
+        x = self.act(x)
+        
+        return x
+
+
+class FeedForward(torch.nn.Module):
+    r"""FeedForward module
+
+    Args:
+        dim_in (int): input dimension.
+        dim_ff (int): middle dimension.
+        dim_out (int, optional): output dimension. Defaults to None, which means dim_in = dim_out.
+        dtype (optional): Defaults to torch.half.
+        init_mean (float, optional): mean of :math:`\mathbf{W}\sim\mathcal{N}(\text{mean}, \text{std}^2)` for fully-connected module used in feed-forward layer. Defaults to 0.
+        init_std (float, optional): std of :math:`\mathbf{W}\sim\mathcal{N}(\text{mean}, \text{std}^2)` for fully-connected module used in feed-forward layer. Defaults to 0.02.
+        bias (bool, optional): whether to use bias term in fully-connected layers used in feed-forward module. Defaults to False.
+        activate_fn (str, optional): Defaults to `gated_gelu`.
+        dropout_p (int, optional): Defaults to 0.
+    """
+
+    def __init__(self,
+                 dim_in : int, 
+                 dim_ff : int,
+                 dim_out : int = None,
+                 dtype = torch.half, 
+                 int8 = False,
+                 init_mean = 0.0, 
+                 init_std = 0.02,
+                 bias = False,
+                 activate_fn = "gated_gelu",
+                 length_scale : bool = False,
+                 dropout_p = 0,
+        ):
+
+        super().__init__()
+
+        if activate_fn.startswith("gated_"):
+            self.w_in = DenseGatedACT(
+                dim_in = dim_in,
+                dim_ff = dim_ff,
+                activate_fn = activate_fn[6:],
+                dtype = dtype,
+                int8 = int8,
+                init_mean = init_mean,
+                init_std = init_std,
+                bias = bias,
+                length_scale = length_scale,
+            )
+        else:
+            self.w_in = DenseACT(
+                dim_in = dim_in,
+                dim_ff = dim_ff,
+                activate_fn = activate_fn,
+                dtype = dtype,
+                int8 = int8,
+                init_mean = init_mean,
+                init_std = init_std,
+                bias = bias,
+                length_scale = length_scale,
+            )
+
+        if dropout_p:
+            self.dropout = torch.nn.Dropout(dropout_p)
+        else:
+            self.dropout = None
+
+        if dim_out is None:
+            dim_out = dim_in
+
+        self.dim_ff = dim_ff
+        self.dim_out = dim_out
+
+        self.w_out = CPM3Linear(
+            dim_in = dim_ff,
+            dim_out = dim_out,
+            length_scale = length_scale,
+            length_scale_before = True,
+            dtype = dtype,
+            int8 = int8,
+            bias = bias,
+        )
+
+        self.int8 = int8
+        self.length_scale = length_scale
+
+    def forward(self, x : torch.Tensor):
+        """ 
+        Args:
+            x (:obj:`torch.Tensor` of shape ``(batch, seq_len, dim_in)``): The input of feed-forward module.
+
+        Return:
+            :obj:`torch.Tensor` of shape ``(batch, seq_len, dim_out)``: The output of feed-forward module.
+        """
+        x = self.w_in(x)
+
+        if self.dropout is not None:
+            x = self.dropout(x)
+
+        x = self.w_out(x)
+
+        return x
+
+
+class CPM3FFN(torch.nn.Module):
+    """ The whole feed-forward block. A sequence of operation. Consists of layernorm, feed-forward and residual connection.
+
+    Args:
+        dim_model (int): main dimension of modules in transformer blocks.
+        dim_ff (int): dim_ff used in :py:class:`model_center.layer.FeedForward`.
+        dtype (optional): Defaults to torch.half.
+        norm_init_var (float, optional): init_var used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1.0.
+        norm_bias (bool, optional): bias used in :py:class:`model_center.layer.LayerNorm`. Defaults to False.
+        norm_eps (float, optional): eps used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1e-5.
+        ffn_init_mean (float, optional): init_mean used in :py:class:`model_center.layer.FeedForward`. Defaults to 0.0.
+        ffn_init_std (float, optional): init_std used in :py:class:`model_center.layer.FeedForward`. Defaults to 0.02.
+        ffn_bias (bool, optional): bias used in :py:class:`model_center.layer.FeedForward`. Defaults to False.
+        ffn_activate_fn (str, optional): activate_fn used in :py:class:`model_center.layer.FeedForward`. Defaults to "gated_gelu".
+        post_layer_norm (bool, optional): whether to use post-layernorm. Defaults to False, which means pre-layernorm.
+        dropout_p (float, optional): Defaults to 0.
+    """
+
+    def __init__(self, 
+                 dim_model : int, 
+                 dim_ff : int,
+                 dtype = torch.half, 
+                 int8 = False,
+                 norm_init_var : float = 1.0,
+                 norm_bias : bool = False,
+                 norm_eps : float = 1e-5, 
+                 ffn_init_mean : float = 0.0, 
+                 ffn_init_std : float = 0.02,
+                 ffn_bias : bool = False,
+                 ffn_activate_fn : str = "gated_gelu",
+                 post_layer_norm : bool = False,
+                 length_scale : bool = False,
+                 dropout_p : float = 0,
+                ):
+        super().__init__()
+
+        self.layernorm_before_ffn = CPM3LayerNorm(
+            dim_norm = dim_model, 
+            bias = norm_bias, 
+            dtype = dtype,
+            eps = norm_eps, 
+            init_var = norm_init_var,
+        )
+
+        self.ffn = FeedForward(
+            dim_in = dim_model, 
+            dim_ff = dim_ff, 
+            dim_out = dim_model, 
+            dtype = dtype, 
+            int8 = int8,
+            init_mean = ffn_init_mean, 
+            init_std = ffn_init_std,
+            bias = ffn_bias,
+            activate_fn = ffn_activate_fn,
+            length_scale = length_scale,
+        )
+
+        if dropout_p:
+            self.dropout = torch.nn.Dropout(dropout_p)
+        else:
+            self.dropout = None
+
+        self.post_layer_norm = post_layer_norm
+
+    def forward(self,
+                hidden_states : torch.Tensor,
+               ):
+        """ 
+        Args:
+            hidden_states (:obj:`torch.Tensor` of shape ``(batch, seq_self, dim_model)``): Hidden states before feed forward layer.
+
+        Return:
+            :obj:`torch.Tensor` of shape ``(batch, seq_self, dim_model)``: The output of feed-forward block
+
+        """ 
+        x = self.layernorm_before_ffn(hidden_states)
+        if self.post_layer_norm:
+            hidden_states = x
+        x = self.ffn(x)
+        if self.dropout is not None:
+            x = self.dropout(x)
+        hidden_states = hidden_states + x
+        return hidden_states
\ No newline at end of file
