@@ -1,4 +1,3 @@
-from time import clock_settime
 from transformers.models.clip.modeling_clip import *
 import torch.nn as nn
 import torch
@@ -8,7 +7,6 @@ import os
 from flagai.model.base_model import BaseModel
 
 from .modeling_berts import BertSeriesConfig, RobertaSeriesConfig, BertSeriesModelWithTransformation, RobertaSeriesModelWithTransformation
-from transformers.models.bert.tokenization_bert import BertTokenizer
 
 STUDENT_CONFIG_DICT = {
     'hfl/chinese-roberta-wwm-ext': BertSeriesConfig,
@@ -247,7 +245,7 @@ class CLIPHF(CLIPPreTrainedModel):
                padding="max_length",
                truncation=True,
                max_length=77):
-        device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
+        device = next(self.text_model.parameters()).device
         text = tokenizer(text,
                          truncation=True,
                          max_length=77,
