@@ -6,9 +6,10 @@ import torch
 import torch.nn.functional as F
 from flagai.model.predictor.utils import viterbi_decode, decode_labels, bert_beamsearch,\
     t5_random_sample, gpt_random_sample, \
-    t5_beamsearch, gpt_beamsearch, bert_random_sample, glm_beamsearch, glm_random_sample, cpm_beamsearch
+    t5_beamsearch, gpt_beamsearch, bert_random_sample, glm_beamsearch, glm_random_sample, cpm_beamsearch, alm_beamsearch, alm_random_sample
 from typing import List, Union, Dict, Tuple, Any
 from flagai.model.predictor.gpt import gpt_random_sample_use_cache
+from flagai.model.predictor.simctg import contrastive_search
 from flagai.model.mm.Sampler import DDIMSampler, PLMSSampler
 import os
 from PIL import Image
@@ -16,9 +17,9 @@ from tqdm import trange, tqdm
 import time
 from contextlib import contextmanager, nullcontext
 from einops import rearrange
+from torch.cuda.amp import autocast as autocast
 
 class Predictor:
-
     def __init__(self, model, tokenizer=None):
         """
         Args:
@@ -230,6 +231,10 @@ class Predictor:
             #assert "seq2seq" in self.class_name.lower(), "this function only support seq2seq task"
             return glm_beamsearch(self.model, self.tokenizer, text,
                                   out_max_length, beam_size)
+        if "alm" in self.class_name.lower():
+            #assert "seq2seq" in self.class_name.lower(), "this function only support seq2seq task"
+            return alm_beamsearch(self.model, self.tokenizer, text,
+                                  out_max_length, beam_size)
         if "bert" in self.class_name.lower():
             assert "seq2seq" in self.class_name.lower(
             ), "this function only support seq2seq task"
@@ -255,7 +260,6 @@ class Predictor:
                                   out_max_length=out_max_length,
                                   beam_size=beam_size)
         elif "cpm" in self.class_name.lower():
-            print('self.tokenizer', self.tokenizer)
             return cpm_beamsearch(self.model,
                                   self.tokenizer,
                                   text,
@@ -267,6 +271,31 @@ class Predictor:
             import os
             os._exit(0)
 
+    def predict_generate_contrastive_search(self,
+                                            text: str,
+                                            input_max_length: int = 256,
+                                            out_max_length: int = 100,
+                                            beam_size: int = 1) -> str:
+        """
+        Args:
+          text: The input text.
+          input_max_length: The max length of input text.
+          out_max_length: The max length of output text.
+          beam_size: The beam size.
+        """
+        self.model.eval()
+        if "gpt" in self.class_name.lower() or "opt" in self.class_name.lower():
+            return contrastive_search(self.model,
+                                      self.tokenizer,
+                                      text,
+                                      input_max_length=input_max_length,
+                                      out_max_length=out_max_length,
+                                      beam_size=beam_size)
+        else:
+            print("Unsupported decoding mode")
+            import os
+            os._exit(0)
+
     def predict_generate_randomsample(self,
                                       text: str,
                                       input_max_length: int = 256,
@@ -293,16 +322,21 @@ class Predictor:
                                     device)
 
         elif "gpt" in self.class_name.lower(
-        ) or "opt" in self.class_name.lower():
+        ) or "opt" in self.class_name.lower() \
+            or "galactica" in self.class_name.lower():
             return gpt_random_sample_use_cache(self.model, self.tokenizer,
                                                text, input_max_length,
                                                out_max_length, top_k, top_p,
                                                repetition_penalty, temperature,
                                                device)
-        elif "glm" in self.class_name.lower():
+        elif "glm" in self.class_name.lower() or "alm" in self.class_name.lower():
             return glm_random_sample(self.model, self.tokenizer, text,
                                      out_max_length, top_k, top_p,
                                      repetition_penalty, temperature, device)
+        elif "alm" in self.class_name.lower():
+            return alm_random_sample(self.model, self.tokenizer, text,
+                                     out_max_length, top_k, top_p,
+                                     repetition_penalty, temperature, device)
 
         elif "bert" in self.class_name.lower():
             assert "seq2seq" in self.class_name.lower(
@@ -335,7 +369,8 @@ class Predictor:
                                 f: int = 8,
                                 scale: float = 7.5,
                                 from_file: str = None,
-                                seed: int = 34234):
+                                seed: int = 34234,
+                                fp16: bool = False):
         from torchvision.utils import make_grid
         from pytorch_lightning import seed_everything
         from flagai.model.predictor.utils import chunk, check_safety, get_safety_checker
@@ -357,7 +392,6 @@ class Predictor:
         C: channels of images, 4 for colored images
         """
         seed_everything(seed)
-
         assert "diffusion" in self.class_name.lower()
         device = next(self.model.parameters()).device
         if plms:
@@ -415,9 +449,9 @@ class Predictor:
                             unconditional_conditioning=uc,
                             eta=ddim_eta,
                             x_T=start_code)
-
-                        x_samples_ddim = self.model.decode_first_stage(
-                            samples_ddim)
+                        with autocast():
+                            x_samples_ddim = self.model.decode_first_stage(
+                                samples_ddim)
                         x_samples_ddim = torch.clamp(
                             (x_samples_ddim + 1.0) / 2.0, min=0.0, max=1.0)
                         x_samples_ddim = x_samples_ddim.cpu().permute(
