@@ -12,6 +12,12 @@ try:
 except Exception:
     pass
 
+try:
+    import bmtrain as bmt
+except:
+    pass
+
+
 import torch
 import argparse
 import os
@@ -127,6 +133,7 @@ class Trainer():
         fp16=False,
         clip_grad=1.0,
         checkpoint_activations=False,
+        tokenizer=None,
 
         # model checkpointing
         save_dir='checkpoints',  # 'Output directory to save checkpoints to.')
@@ -152,6 +159,7 @@ class Trainer():
         deepspeed_config=None,
         model_parallel_size=1,
         training_script="train.py",
+        optimizer_type='adam',
     ):
 
         if timers is not None:
@@ -160,7 +168,7 @@ class Trainer():
             self.timers = Timers()
         self.env_type = env_type
         if env_type not in set(
-            ["deepspeed", 'pytorch', 'pytorchDDP', 'deepspeed+mpu']):
+            ["deepspeed", 'pytorch', 'pytorchDDP', 'deepspeed+mpu', 'bmtrain']):
             raise Exception("Not supported env_type!!!!")
         os.environ["ENV_TYPE"] = env_type
         self.experiment_name = experiment_name
@@ -176,6 +184,9 @@ class Trainer():
 
         self.log_interval = log_interval
         self.eval_interval = eval_interval
+        self.tokenizer = tokenizer
+
+        self.optimizer_type = optimizer_type
 
         # model checkpointing
         self.save_dir = save_dir
@@ -206,7 +217,7 @@ class Trainer():
         self.hostfile = hostfile
         self.training_script = training_script
 
-        if 'deepspeed' in self.env_type or self.env_type == 'pytorchDDP':
+        if self.env_type != 'pytorch':
             training_paras = self.get_dist_args()
             # Implement for AutoLaunch
             # >>> python train.py # will call get_dist_args()
@@ -245,8 +256,6 @@ class Trainer():
         self.not_call_launch = ds_args.not_call_launch
         self.rank = int(os.environ.get('RANK', 0))
         self.world_size = int(os.environ.get('WORLD_SIZE', 1))
-        self.master_addr = os.environ.get('MASTER_ADDR', '127.0.0.1')
-        self.master_port = os.environ.get('MASTER_PORT', '17500')
         log_dist("not_call_launch: {}".format(ds_args.not_call_launch))
         return []
 
@@ -264,7 +273,7 @@ class Trainer():
         if self.env_type == 'pytorch':
             log_dist('No need to initialize')
             return
-        if self.env_type in ['deepspeed', 'deepspeed+mpu', 'pytorchDDP']:
+        if self.env_type in ['deepspeed', 'deepspeed+mpu', 'pytorchDDP', 'bmtrain']:
             torch.backends.cudnn.enabled = False
             # Manually set the device ids.
             device = self.rank % torch.cuda.device_count()
@@ -280,11 +289,17 @@ class Trainer():
             log_dist(
                 "init method {}, rank {}, device {}, local_rank {}.".format(
                     init_method, self.rank, device, self.local_rank))
-            torch.distributed.init_process_group(
-                backend='nccl',  # gloo
-                world_size=self.world_size,
-                rank=self.rank,
-                init_method=init_method)
+            if self.env_type == 'bmtrain':
+                # self.get_env_args()
+                bmt.init_distributed(
+                    seed=self.seed,
+                    init_method=init_method)
+            else:
+                torch.distributed.init_process_group(
+                    backend='nccl',  # gloo
+                    world_size=self.world_size,
+                    rank=self.rank,
+                    init_method=init_method)
         # Set the model-parallel / data-parallel communicators.
         if self.env_type == 'deepspeed+mpu':
             os.environ["MODEL_PARALLEL_SIZE"] = str(self.model_parallel_size)
@@ -302,7 +317,8 @@ class Trainer():
                 log_dist(e)
                 log_dist("No mpu is installed! No model parallel is used")
             log_dist("initialize eviroments succesed")
-        self.set_seed(self.seed)
+        if self.env_type != 'bmtrain':
+            self.set_seed(self.seed)
 
     def get_dataloader(self, dataset, collate_fn, shuffle=False):
         """ initilize the dataloader"""
@@ -320,12 +336,24 @@ class Trainer():
         else:
             if self.env_type == 'deepspeed+mpu':
                 rank = mpu.get_model_parallel_src_rank()
-                print("*"*80)
-                print("local rank",self.rank, "model rank", rank)
-                print("*"*80)
+                data_rank = mpu.get_data_parallel_rank()
+                log_dist("*"*80)
+                log_dist(f"local rank {self.rank} src rank  {rank} data rank {data_rank}")
+                log_dist("*"*80)
                 sampler = torch.utils.data.distributed.DistributedSampler(
                     dataset,
-                    # num_replicas=num_replicas,
+                    num_replicas=self.world_size//self.model_parallel_size,
+                    rank=data_rank,
+                    shuffle=shuffle)
+            elif self.env_type == 'bmtrain':
+                print("*"*80)
+                print("local rank", self.rank, "world_size", self.world_size, "bmt rank", bmt.rank())
+                print("*"*80)
+                num_replicas = self.world_size
+                rank = self.rank
+                sampler = torch.utils.data.distributed.DistributedSampler(
+                    dataset,
+                    num_replicas=num_replicas,
                     rank=rank,
                     shuffle=shuffle)
             else:
@@ -403,7 +431,6 @@ class Trainer():
                                  load_type=self.load_type)
         """Train the model."""
         # Turn on training mode which enables dropout.
-        model.train()
         if self.fp16 and self.env_type == 'pytorchDDP':
             log_dist(
                 "Warning: The pytorchDDP plus FP16 may not working togather!!!"
@@ -422,6 +449,8 @@ class Trainer():
 
         elif self.env_type == 'pytorch':
             model.to(self.pytorch_device)
+        elif self.env_type == 'bmtrain':
+            model = bmt.BMTrainModelWrapper(model)
         else:
             model.cuda(torch.device('cuda', self.local_rank))
         if self.fp16:
@@ -434,6 +463,11 @@ class Trainer():
             param_groups = param_groups[0]['params']
 
         if optimizer is None and 'deepspeed' not in self.env_type and self.epochs > 0:
+            # if self.env_type == 'bmtrain':
+            #     optimizer = bmt.optim.AdamOptimizer(model.parameters(), 
+            #                                    weight_decay=self.weight_decay, lr=5e-6)
+            #     # optimizer = torch.optim.Adam(model.parameters(), lr=2e-4)
+            # else:
             optimizer = get_optimizer(
                 param_groups=param_groups,
                 lr=self.lr,
@@ -441,17 +475,28 @@ class Trainer():
                 cpu_optimizer=False,
                 cpu_torch_adam=False,
                 fp16=self.fp16,
-                optimizer='adam')  # if not self.fp16 else 'adafactor')
+                optimizer=self.optimizer_type)  # if not self.fp16 else 'adafactor')
 
         if lr_scheduler == None and optimizer != None and self.warm_up > 0 and 'deepspeed' not in self.env_type and self.epochs > 0:
+            if self.env_type == 'bmtrain':
+                lr_scheduler = bmt.lr_scheduler.Noam(optimizer,
+                                                    start_lr =self.lr, 
+                                                    warmup_iter =int(self.warm_up * self.epochs * len(train_dataloader)),
+                                                    end_iter = self.epochs * len(train_dataloader)
+                                                )
+            else:
+                lr_scheduler = AnnealingLR(
+                    optimizer,
+                    start_lr=self.lr,
+                    warmup_iter=int(self.warm_up * self.epochs *
+                                    len(train_dataloader)),
+                    decay_style='linear',
+                    num_iters=self.epochs * len(train_dataloader))
 
-            lr_scheduler = AnnealingLR(
-                optimizer,
-                start_lr=self.lr,
-                warmup_iter=int(self.warm_up * self.epochs *
-                                len(train_dataloader)),
-                decay_style='linear',
-                num_iters=self.epochs * len(train_dataloader))
+        if  self.env_type == 'bmtrain':
+            bmt.synchronize()
+
+        model.train()
 
         if 'deepspeed' in self.env_type:
             # initialize the deepspeed
@@ -483,18 +528,14 @@ class Trainer():
         if len(self.metric_methods) > 0:
             best_score = -best_score
         for epoch in range(self.epochs):
-            # log_dist('working on epoch {} ...'.format(epoch), [0])
-            # Set the data loader epoch to shuffle the index iterator.
-            # if self.env_type == 'deepspeed+mpu':
-            #     if mpu.get_model_parallel_rank() == 0:
-            #         train_dataloader.sampler.set_epoch(epoch + self.world_size)
+            print("epoch "+str(epoch))
             if self.env_type != 'pytorch':
                 train_dataloader.sampler.set_epoch(epoch + self.world_size)
 
             # For all the batches in the dataset.
             for iteration_, batch in enumerate(train_dataloader):
                 # Train for one step.
-                if 'deepspeed' in self.env_type or self.env_type == 'pytorchDDP':
+                if 'pytorch' != self.env_type:
                     batch = {
                         x: batch[x].to(torch.device('cuda', self.local_rank))
                         for x in batch if x not in ['uid', 'meta', 'mode']
@@ -512,6 +553,11 @@ class Trainer():
                 elif self.env_type == 'pytorch':
                     lm_loss, _ = self.train_step_pytorch(
                         batch, model, optimizer, lr_scheduler)
+                
+                elif self.env_type == 'bmtrain':
+                    lm_loss, _ = self.train_step_bmtrain(
+                        batch, model, optimizer, lr_scheduler)
+
                 else:
                     lm_loss, _ = self.train_step_deepspeed(batch,
                                                            model,
@@ -520,7 +566,10 @@ class Trainer():
                                                            single_step=True)
                     dist.barrier()
                 if lm_loss is not None:
-                    total_lm_loss += lm_loss.data.detach().float()
+                    if not isinstance(lm_loss, float):
+                        total_lm_loss += lm_loss.data.detach().float()
+                    else:
+                        total_lm_loss += lm_loss
 
                 # Logging.
                 if (self.iteration + 1) % self.log_interval == 0:
@@ -528,7 +577,10 @@ class Trainer():
                         learning_rate = optimizer.param_groups[0]['lr']
                     else:
                         learning_rate = model.optimizer.param_groups[0]['lr']
-                    avg_lm_loss = total_lm_loss.item() / self.log_interval
+                    if self.env_type == 'bmtrain':
+                        avg_lm_loss = total_lm_loss / self.log_interval
+                    else:
+                        avg_lm_loss = total_lm_loss.item() / self.log_interval
                     elapsed_time = self.timers('interval time').elapsed()
                     self.report_iteration_metrics(
                         optimizer, learning_rate, avg_lm_loss,
@@ -588,7 +640,6 @@ class Trainer():
                                     save_dir=self.save_dir,
                                     save_rng=self.save_rng)
                 self.iteration += 1
-
                 # Checkpointing at the end of each epoch.
 
         # Evaluation #todo add train_args
@@ -783,6 +834,34 @@ class Trainer():
             mems = []
             reduced_loss = None
         return reduced_loss, mems
+    
+    def train_step_bmtrain(self,
+                             data,
+                             model,
+                             optimizer,
+                             lr_scheduler,
+                             mems=None,
+                             single_step=False): #bmt has loss scale, which need to be checked repetition
+        """Single training step."""
+        optim_manager = bmt.optim.OptimManager(loss_scale=1024)
+        optim_manager.add_optimizer(optimizer, lr_scheduler)
+        optim_manager.zero_grad()
+        self.timers('forward').start()
+        model_output = model(**data)
+        self.timers('forward').stop()
+        logits = model_output['logits']
+        loss = model_output['loss']
+        lm_loss = bmt.sum_loss(loss).item()
+        self.timers('backward').start()
+        try:
+            optim_manager.backward(loss)
+        except:
+            loss.backward()
+        self.timers('backward').stop()
+        self.timers('optimizer').start()
+        optim_manager.step()
+        self.timers('optimizer').stop()
+        return lm_loss, mems
 
     def forward_step(self, data, model, mems=None):
         """Simple forward step. """
@@ -812,7 +891,7 @@ class Trainer():
         if 'deepspeed' in self.env_type:
             model.backward(loss)
         else:
-            # optimizer.zero_grad()
+            optimizer.zero_grad()
             if hasattr(optimizer, 'backward'):
                 optimizer.backward(loss, update_master_grads=False)
             else:
@@ -881,7 +960,6 @@ class Trainer():
                  forward_step_func=None,
                  verbose=False):
         """Evaluation."""
-
         # Turn off checkpoint_activations
         tmp_checkpoint_activations = None
         tmp_model = model
@@ -897,12 +975,12 @@ class Trainer():
 
         mems = None
         metrics = [0. for _ in range(len(self.metric_methods))]
-
         with torch.no_grad():
             assert data_loader is not None, "val loader is not None."
             all_logits = []
             all_labels = []
             all_losses = []
+            prev = False
             for data_iterator in data_loader:
                 # Forward evaluation.
                 meta = data_iterator.get('meta', None)
@@ -931,7 +1009,7 @@ class Trainer():
                     deepspeed.checkpointing.reset()
                 logits = step_output['logits']
                 lm_loss = step_output['loss']
-
+                
                 if 'labels' in data_iterator:
                     labels = data_iterator['labels']
                 else:
@@ -943,15 +1021,22 @@ class Trainer():
                         all_logits.extend(batch_preds)
                         all_labels.extend(batch_labels)
                     else:
+                        if prev or logits.size(0) != 1:
+                            logits = torch.argmax(logits, dim=1)
+                            prev = True
                         all_logits.append(logits)
                         all_labels.append(labels)
+                        pass
                 all_losses.append(lm_loss.view(1))
-            if self.env_type != 'pytorch' and len(self.metric_methods) != 0:
+            # size of all_logits: (1, n)
+            if len(self.metric_methods) != 0 and all_logits[0].size() != all_logits[-1].size():
+                pd = (all_logits[0].size(0)-all_logits[-1].size(0), all_logits[0].size(1)-all_logits[-1].size(1))
+                all_logits[-1] = torch.nn.functional.pad(all_logits[-1], pd ,"constant",0)
+            all_losses = torch.cat(all_losses, dim=0)
+            if len(self.metric_methods) != 0:
                 all_logits = torch.cat(all_logits, dim=0)
                 all_labels = torch.cat(all_labels, dim=0)
 
-            all_losses = torch.cat(all_losses, dim=0)
-
             if self.env_type == 'pytorchDDP' or self.env_type == 'deepspeed':
                 if len(self.metric_methods) != 0:
                     all_logits = self._gather_all(all_logits)
@@ -969,11 +1054,10 @@ class Trainer():
 
             for i in range(len(self.metric_methods)):
                 eval_method = self.metric_methods[i][1]
-                metrics[i] += eval_method(all_logits, all_labels, meta=meta)
+                metrics[i] += eval_method(all_logits, all_labels, meta=meta, tokenizer=self.tokenizer)
 
         # Move model back to the train mode.
 
-        # model.train()
         tmp_model.train()
         # recover the settings for checkpoint_activations
         if hasattr(tmp_model,
@@ -1021,6 +1105,7 @@ class Trainer():
         verbose=False,
     ):
         """Helper function to evaluate and dump results on screen."""
+        
         eval_dict = self.evaluate(forward_step_func=forward_step_func,
                                   data_loader=data_loader,
                                   model=model,
